@@ -260,7 +260,10 @@ class ConjectureRunner(object):
             if self.valid_examples >= self.settings.max_examples:
                 self.exit_with(ExitReason.max_examples)
             if self.call_count >= max(
-                self.settings.max_iterations, self.settings.max_examples
+                self.settings.max_examples * 10,
+                # We have a high-ish default max iterations, so that tests
+                # don't become flaky when max_examples is too low.
+                1000
             ):
                 self.exit_with(ExitReason.max_iterations)
 
@@ -542,7 +545,6 @@ class ConjectureRunner(object):
 
         This causes us to enter "shrinking mode" there and thus reduce
         the size of the generated data.
-
         """
         initial = len(result)
         if data.depth * 2 >= MAX_DEPTH or data.index >= self.cap:
@@ -581,7 +583,6 @@ class ConjectureRunner(object):
         which the last failure came from but we don't enforce that). We
         then take a random sampling of the remainder and try those. Any
         examples that are no longer interesting are cleared out.
-
         """
         if self.has_existing_examples():
             self.debug('Reusing examples from database')
@@ -660,8 +661,7 @@ class ConjectureRunner(object):
                 HealthCheck.large_base_example
             )
 
-        if self.settings.perform_health_check:
-            self.health_check_state = HealthCheckState()
+        self.health_check_state = HealthCheckState()
 
         count = 0
         while not self.interesting_examples and (
@@ -780,7 +780,6 @@ class ConjectureRunner(object):
 
         We may find one or more examples with a new interesting_origin
         during the shrink process. If so we shrink these too.
-
         """
         if (
             Phase.shrink not in self.settings.phases or
@@ -864,7 +863,6 @@ class ConjectureRunner(object):
 
         This is purely an optimisation to try to reduce the number of tests we
         run. "return True" would be a valid but inefficient implementation.
-
         """
         node_index = 0
         n = len(buffer)
@@ -971,8 +969,8 @@ class SampleSet(object):
     values to their index in an array. Sampling uniformly at random then
     becomes simply a matter of sampling from the array, but we can use
     the index for efficient lookup to add and remove values.
-
     """
+
     __slots__ = ('__values', '__index')
 
     def __init__(self):
@@ -1081,7 +1079,6 @@ class TargetSelector(object):
     4. Among the interesting deduplicated coverage targets we essentially
        round-robin between them, but with a more consistent distribution than
        uniformly at random, which is important particularly for short runs.
-
     """
 
     def __init__(self, random):
@@ -1200,7 +1197,6 @@ class Shrinker(object):
 
     The expected usage pattern is that this is only ever called from within the
     engine.
-
     """
 
     def __init__(self, engine, initial, predicate):
@@ -1211,7 +1207,6 @@ class Shrinker(object):
 
         Note that initial is a ConjectureData object, and predicate
         takes ConjectureData objects.
-
         """
         self.__engine = engine
         self.__predicate = predicate
@@ -1263,7 +1258,6 @@ class Shrinker(object):
 
         This method is "mostly idempotent" - calling it twice is unlikely to
         have any effect, though it has a non-zero probability of doing so.
-
         """
         # We assume that if an all-zero block of bytes is an interesting
         # example then we're not going to do better than that.
@@ -1288,9 +1282,7 @@ class Shrinker(object):
 
         This method iterates to a fixed point and so is idempontent - calling
         it twice will have exactly the same effect as calling it once.
-
         """
-
         run_expensive_shrinks = False
 
         prev = None
@@ -1330,6 +1322,7 @@ class Shrinker(object):
             if not run_expensive_shrinks:
                 continue
 
+            self.shrink_offset_pairs()
             self.interval_deletion_with_block_lowering()
             self.pass_to_interval()
             self.reorder_bytes()
@@ -1399,7 +1392,6 @@ class Shrinker(object):
         there is a one-byte interval starting at right place for ls[i] and try
         to replace it with that. This will successfully replace ls[i] with []
         as desired.
-
         """
         i = 0
         while i < len(self.shrink_target.examples):
@@ -1437,7 +1429,6 @@ class Shrinker(object):
         This is pretty expensive - it takes O(len(intervals)^2) - so we run it
         late in the process when we've got the number of intervals as far down
         as possible.
-
         """
         i = 0
         while i < len(self.shrink_target.examples):
@@ -1466,7 +1457,6 @@ class Shrinker(object):
         If the shrink target has changed since i was last checked, will
         attempt to calculate if an equivalent block in a previous shrink
         target was marked as shrinking.
-
         """
         if not self.__shrinking_prefixes:
             return False
@@ -1512,7 +1502,6 @@ class Shrinker(object):
         shape of the blocks changes as a result of this bouncing behaviour),
         but it fails fast when it doesn't work and gets us out of a really
         nastily slow case when it does.
-
         """
         if len(self.__changed_blocks) <= 1:
             return
@@ -1541,6 +1530,68 @@ class Shrinker(object):
 
         minimize_int(offset, reoffset)
 
+    def shrink_offset_pairs(self):
+        """Lower any two blocks offset from each other the same ammount.
+
+        Before this shrink pass, two blocks explicitly offset from each
+        other would not get minimized properly:
+         >>> b = st.integers(0, 255)
+         >>> find(st.tuples(b, b), lambda x: x[0] == x[1] + 1)
+        (149,148)
+
+        This expensive (O(n^2)) pass goes through every pair of non-zero
+        blocks in the current shrink target and sees if the shrink
+        target can be improved by applying an offset to both of them.
+        """
+        self.debug('Shrinking offset pairs.')
+
+        current = [self.shrink_target.buffer[u:v] for u, v in self.blocks]
+
+        def int_from_block(i):
+            return int_from_bytes(current[i])
+
+        def block_len(i):
+            u, v = self.blocks[i]
+            return v - u
+
+        # Try reoffseting every pair
+        def reoffset_pair(pair, o):
+            n = len(self.blocks)
+            # Number of blocks may have changed, need to validate
+            valid_pair = [p for p in pair if p < n and int_from_block(p) > 0]
+
+            if len(valid_pair) < 2:
+                return
+
+            m = min([int_from_block(p) for p in valid_pair])
+
+            new_blocks = [self.shrink_target.buffer[u:v]
+                          for u, v in self.blocks]
+            for i in valid_pair:
+                new_blocks[i] = int_to_bytes(
+                    int_from_block(i) + o - m, block_len(i))
+            buffer = hbytes().join(new_blocks)
+            return self.incorporate_new_buffer(buffer)
+
+        i = 0
+        while i < len(self.blocks):
+            if not self.is_shrinking_block(i) and int_from_block(i) > 0:
+                j = i + 1
+                while j < len(self.shrink_target.blocks):
+                    block_val = int_from_block(j)
+                    i_block_val = int_from_block(i)
+                    if not self.is_shrinking_block(j) \
+                       and block_val > 0 and i_block_val > 0:
+                        offset = min(int_from_block(i),
+                                     int_from_block(j))
+                        # Save current before shrinking
+                        current = [self.shrink_target.buffer[u:v]
+                                   for u, v in self.blocks]
+                        minimize_int(
+                            offset, lambda o: reoffset_pair((i, j), o))
+                    j += 1
+            i += 1
+
     def mark_shrinking(self, blocks):
         """Mark each of these blocks as a shrinking block: That is, lowering
         its value lexicographically may cause less data to be drawn after."""
@@ -1608,7 +1659,6 @@ class Shrinker(object):
         result in losing the last element of y, and the test would start
         passing. But if we were to replace this with [a, b], [c, d] with c != 0
         then deleting a or b would work.
-
         """
         count = 0
         while count < 10:
@@ -1661,9 +1711,7 @@ class Shrinker(object):
         handled by the interval_deletion_with_block_lowering pass - but will be
         effective when there is a large amount of redundant data after the
         block to be lowered.
-
         """
-
         initial_attempt = bytearray(self.shrink_target.buffer)
         for i in blocks:
             if i >= len(self.blocks):
@@ -1715,7 +1763,6 @@ class Shrinker(object):
         each interval individually. The common case is that all data drawn and
         rejected can just be thrown away immediately in one block, so this pass
         will be much faster than trying each one individually when it works.
-
         """
         if not self.shrink_target.has_discards:
             return
@@ -1728,6 +1775,8 @@ class Shrinker(object):
             ):
                 discarded.append((ex.start, ex.end))
 
+        assert discarded
+
         attempt = bytearray(self.shrink_target.buffer)
         for u, v in reversed(discarded):
             del attempt[u:v]
@@ -1763,7 +1812,6 @@ class Shrinker(object):
 
         This pass will attempt to delete adjacent pairs of calls to shorten the
         loop.
-
         """
         self.debug('greedy interval deletes')
         i = 0
@@ -1865,9 +1913,7 @@ class Shrinker(object):
         It is also useful for when that duplication is accidental and the value
         of the blocks doesn't matter very much because it allows us to replace
         more values at once.
-
         """
-
         self.debug('Simultaneous shrinking of duplicated blocks')
 
         def canon(b):
@@ -1911,7 +1957,6 @@ class Shrinker(object):
         assert x < 10
 
         then in our shrunk example, x = 10 rather than say 97.
-
         """
         self.debug('Shrinking of individual blocks')
         i = 0
@@ -1953,7 +1998,6 @@ class Shrinker(object):
         can then try to delete the lost bytes (see try_shrinking_blocks for how
         this works), taking us immediately to [1, 1]. This is a less important
         role for this pass, but still significant.
-
         """
         self.debug('Reordering blocks')
         block_lengths = sorted(self.shrink_target.block_starts, reverse=True)
@@ -2010,9 +2054,7 @@ class Shrinker(object):
         shrinks, so it's important that it runs late in the process when the
         example size is small and most of the blocks that can be zeroed have
         been.
-
         """
-
         self.debug('Lowering blocks while deleting intervals')
         i = 0
         while i < len(self.intervals):
@@ -2070,7 +2112,6 @@ class Shrinker(object):
         but that won't always work because its length heuristic can be wrong if
         the changes to the next block have knock on size changes, while this
         one triggers more reliably.
-
         """
         self.debug('Lowering adjacent pairs of dependent blocks')
         i = 0
@@ -2116,9 +2157,7 @@ class Shrinker(object):
         order pairs), which is O(n^2) in the worst case. In order to offset we
         try to do as much efficient sorting as possible to reduce the number of
         out of order pairs before we get to that stage.
-
         """
-
         free_bytes = []
 
         for i, (u, v) in enumerate(self.blocks):
