@@ -40,16 +40,14 @@ from hypothesis.control import BuildContext
 from hypothesis._settings import Verbosity
 from hypothesis._settings import settings as Settings
 from hypothesis.reporting import report, verbose_report, current_verbosity
-from hypothesis.strategies import just, lists, builds, one_of, runner, \
-    integers
+from hypothesis.strategies import just, one_of, runner, tuples, \
+    fixed_dictionaries
 from hypothesis.vendor.pretty import CUnicodeIO, RepresentationPrinter
 from hypothesis.internal.reflection import proxies, nicerepr
 from hypothesis.internal.conjecture.data import StopTest
 from hypothesis.internal.conjecture.utils import integer_range, \
     calc_label_from_name
 from hypothesis.searchstrategy.strategies import SearchStrategy
-from hypothesis.searchstrategy.collections import TupleStrategy, \
-    FixedKeysDictStrategy
 
 STATE_MACHINE_RUN_LABEL = calc_label_from_name('another state machine step')
 
@@ -147,6 +145,18 @@ class GenericStateMachine(object):
         """Execute a step that has been previously drawn from self.steps()"""
         raise NotImplementedError(u'%r.execute_step()' % (self,))
 
+    def print_start(self):
+        """Called right at the start of printing.
+
+        By default does nothing.
+        """
+
+    def print_end(self):
+        """Called right at the end of printing.
+
+        By default does nothing.
+        """
+
     def print_step(self, step):
         """Print a step to the current reporter.
 
@@ -179,9 +189,7 @@ class GenericStateMachine(object):
             pass
 
         class StateMachineTestCase(TestCase):
-            settings = Settings(
-                min_satisfying_examples=1
-            )
+            settings = Settings()
 
         # We define this outside of the class and assign it because you can't
         # assign attributes to instance method values in Python 2
@@ -223,27 +231,25 @@ class StateMachineRunner(object):
             print_steps = current_verbosity() >= Verbosity.debug
         self.data.hypothesis_runner = state_machine
 
-        stopping_value = 1 - 1.0 / (1 + self.n_steps * 0.5)
+        should_continue = cu.many(
+            self.data, min_size=1, max_size=self.n_steps,
+            average_size=self.n_steps,
+        )
+
         try:
+            if print_steps:
+                state_machine.print_start()
             state_machine.check_invariants()
 
-            steps = 0
-            while True:
-                if steps >= self.n_steps:
-                    stopping_value = 0
-                self.data.start_example(STATE_MACHINE_RUN_LABEL)
-                if not cu.biased_coin(self.data, stopping_value):
-                    self.data.stop_example()
-                    break
-                assert steps < self.n_steps
+            while should_continue.more():
                 value = self.data.draw(state_machine.steps())
-                steps += 1
                 if print_steps:
                     state_machine.print_step(value)
                 state_machine.execute_step(value)
-                self.data.stop_example()
                 state_machine.check_invariants()
         finally:
+            if print_steps:
+                state_machine.print_end()
             state_machine.teardown()
 
 
@@ -267,8 +273,7 @@ class Rule(object):
 self_strategy = runner()
 
 
-class Bundle(SearchStrategy):
-
+class BundleReferenceStrategy(SearchStrategy):
     def __init__(self, name):
         self.name = name
 
@@ -277,8 +282,18 @@ class Bundle(SearchStrategy):
         bundle = machine.bundle(self.name)
         if not bundle:
             data.mark_invalid()
-        reference = bundle.pop()
-        bundle.insert(integer_range(data, 0, len(bundle)), reference)
+        return bundle[integer_range(data, 0, len(bundle) - 1)]
+
+
+class Bundle(SearchStrategy):
+
+    def __init__(self, name):
+        self.name = name
+        self.__reference_strategy = BundleReferenceStrategy(name)
+
+    def do_draw(self, data):
+        machine = data.draw(self_strategy)
+        reference = data.draw(self.__reference_strategy)
         return machine.names_to_values[reference.name]
 
 
@@ -415,12 +430,6 @@ def invariant():
     return accept
 
 
-@attr.s()
-class ShuffleBundle(object):
-    bundle = attr.ib()
-    swaps = attr.ib()
-
-
 class RuleBasedStateMachine(GenericStateMachine):
     """A RuleBasedStateMachine gives you a more structured way to define state
     machines.
@@ -448,6 +457,8 @@ class RuleBasedStateMachine(GenericStateMachine):
         self.__printer = RepresentationPrinter(self.__stream)
 
     def __pretty(self, value):
+        if isinstance(value, VarReference):
+            return value.name
         self.__stream.seek(0)
         self.__stream.truncate(0)
         self.__printer.output_width = 0
@@ -534,55 +545,49 @@ class RuleBasedStateMachine(GenericStateMachine):
                     if not bundle:
                         valid = False
                         break
+                    v = BundleReferenceStrategy(v.name)
                 converted_arguments[k] = v
             if valid:
-                strategies.append(TupleStrategy((
-                    just(rule),
-                    FixedKeysDictStrategy(converted_arguments)
-                ), tuple))
+                strategies.append(tuples(
+                    just(rule), fixed_dictionaries(converted_arguments)
+                ))
         if not strategies:
             raise InvalidDefinition(
                 u'No progress can be made from state %r' % (self,)
             )
 
-        for name, bundle in self.bundles.items():
-            if len(bundle) > 1:
-                strategies.append(
-                    builds(
-                        ShuffleBundle, just(name),
-                        lists(integers(0, len(bundle) - 1))))
-
         return one_of(strategies)
 
+    def print_start(self):
+        report(u'state = %s()' % (self.__class__.__name__,))
+
+    def print_end(self):
+        report(u'state.teardown()')
+
     def print_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            return
         rule, data = step
         data_repr = {}
         for k, v in data.items():
             data_repr[k] = self.__pretty(v)
         self.step_count = getattr(self, u'step_count', 0) + 1
-        report(u'Step #%d: %s%s(%s)' % (
-            self.step_count,
+        report(u'%sstate.%s(%s)' % (
             u'%s = ' % (self.upcoming_name(),) if rule.targets else u'',
             rule.function.__name__,
             u', '.join(u'%s=%s' % kv for kv in data_repr.items())
         ))
 
     def execute_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            bundle = self.bundle(step.bundle)
-            for i in step.swaps:
-                bundle.insert(i, bundle.pop())
-            return
         rule, data = step
         data = dict(data)
+        for k, v in list(data.items()):
+            if isinstance(v, VarReference):
+                data[k] = self.names_to_values[v.name]
         result = rule.function(self, **data)
         if rule.targets:
             name = self.new_name()
             self.names_to_values[name] = result
             self.__printer.singleton_pprinters.setdefault(
-                id(result), lambda obj, p, cycle: p.text(name),
+                id(result), lambda obj, p, cycle: p.text(name)
             )
             for target in rule.targets:
                 self.bundle(target).append(VarReference(name))
